/******************************************************************************
  (c) Copyright 2009, 1060 Research Ltd                                        

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  $Id$
 *****************************************************************************/
package org.netkernel.xml.saxon.accessor.xpath2;

import java.util.Map;

import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.s9api.Axis;
import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.XPathCompiler;
import net.sf.saxon.s9api.XPathSelector;
import net.sf.saxon.s9api.XdmAtomicValue;
import net.sf.saxon.s9api.XdmEmptySequence;
import net.sf.saxon.s9api.XdmNode;
import net.sf.saxon.s9api.XdmValue;
import net.sf.saxon.sxpath.IndependentContext;
import net.sf.saxon.trans.XPathException;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer1.representation.IDeterminateStringRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.saxon.util.NamespaceMap;
import org.netkernel.xml.saxon.util.SaxonUtil;

/**
 * Provides a NetKernel 4 accessor to XPath2 using Saxon9.
 * 
 * See
 * <a href="/book/view/book:xml:saxon:book/doc:xml:saxon:xpath2">doc:xml:saxon:xpath2</a>.
 * 
 * @author Chris Cormack
 */
public class XPath2Accessor extends StandardAccessorImpl
{
	public XPath2Accessor()
	{	this.declareThreadSafe();
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	String xpathToEval = context.source("arg:operator",
				IDeterminateStringRepresentation.class).getString();
		XdmNode doc = context.source("arg:operand", XdmNode.class);

		NamespaceMap namespaceMap = null;
		if (context.exists("arg:namespaces"))
		{	namespaceMap = context.source("arg:namespaces", NamespaceMap.class);
		}

		XPathSelector xpath = buildEvaluator(doc, namespaceMap).compile(xpathToEval).load();

		xpath.setContextItem(doc);
		context.createResponseFrom(processResult(xpath.evaluate()));
	}

	private Object processResult(XdmValue value)
	{ 	if (value instanceof XdmEmptySequence)
		{	return null;
		} else if (value instanceof XdmAtomicValue)
		{	return ((XdmAtomicValue) value).getValue();
		} else if (value instanceof XdmNode)
		{	return value;
		} else
		{	return value;
		}
	}

	private XPathCompiler buildEvaluator(XdmNode doc, NamespaceMap namespaceMap)
			throws XPathException
	{
		Processor p = SaxonUtil.getProcessor();

		XPathCompiler xpathCompiler = p.newXPathCompiler();

		if (namespaceMap == null)
		{	// set the namespace context as the root element
			IndependentContext context = ((IndependentContext) xpathCompiler
					.getUnderlyingStaticContext());
			context.setNamespaces((NodeInfo) doc.axisIterator(Axis.CHILD)
					.next().getUnderlyingValue());

			xpathCompiler.declareNamespace("xs", "http://www.w3.org/2001/XMLSchema");
		} else
		{	xpathCompiler.declareNamespace("", namespaceMap
					.getDefaultNamespace());

			Map<String, String> namespaces = namespaceMap.getMap();
			for (String prefix : namespaces.keySet())
			{	xpathCompiler.declareNamespace(prefix, namespaces.get(prefix));
			}
		}

		return xpathCompiler;
	}
}
