/******************************************************************************
  (c) Copyright 2009, 1060 Research Ltd                                        

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  $Id$
 *****************************************************************************/
package org.netkernel.xml.saxon.util;

import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URI;

import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;

import net.sf.saxon.Configuration;
import net.sf.saxon.trans.UnparsedTextURIResolver;
import net.sf.saxon.trans.XPathException;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.xml.util.NKFURIResolver;

/**
 * NetKernel aware URIResolver and UnparsedTextURIResolver for Saxon9. This
 * resolver falls back to the CWU or the operator URI if there is no base,
 * before calling NKFURIResolver.resolve
 * 
 * @author Chris Cormack
 * @author Menzo Windhouwer
 */

public class LocalURIResolver implements URIResolver, UnparsedTextURIResolver {
    INKFRequestContext context;
    NKFURIResolver resolver;

    public LocalURIResolver(INKFRequestContext context) {
        this.context = context;
        this.resolver = new NKFURIResolver(context);
    }

    public Source resolve(String href, String base) throws TransformerException {
        if (base == null) {
            if (context.getCWU() != null) {
                base = context.getCWU().toString();
            } else {
                try { // fallback: set the base to the path of the operator
                    base = context.getThisRequest()
                            .getArgumentValue("operator");
                } catch (Exception e) {
                    throw new net.sf.saxon.trans.XPathException(e);
                }
            }
        }
        return resolver.resolve(href, base);
    }

    public Reader resolve(URI uri, String encoding, Configuration config) throws XPathException {
        try {
            IReadableBinaryStreamRepresentation aStream = context.source(uri.toString(),
                    IReadableBinaryStreamRepresentation.class);

            if (aStream != null) {
                if (encoding == null) {
                    encoding = aStream.getEncoding();
                }
                if (encoding != null) {
                    return new InputStreamReader(aStream.getInputStream(),
                            encoding);
                } else {
                    return new InputStreamReader(aStream.getInputStream());
                }
            } else {
                throw new XPathException(
                        "can't create a Reader for unparsed text URI[" + uri + "]");
            }
        } catch (Exception e) {
            throw new XPathException(e);
        }
    }
}
